@@ -1,6 +1,6 @@
-static char rcsid[] = "$Id: main.cc,v 1.48 2000/01/24 23:01:07 bmah Exp $";
+static char rcsid[] = "$Id: main.cc,v 1.54 2000/05/10 20:54:05 bmah Exp $";
 //
-// $Id: main.cc,v 1.48 2000/01/24 23:01:07 bmah Exp $
+// $Id: main.cc,v 1.54 2000/05/10 20:54:05 bmah Exp $
 //
 // main.cc
 // Bruce A. Mah <bmah@ca.sandia.gov>
@@ -35,6 +35,7 @@ extern "C" {
 #include <sys/types.h>
 #include <time.h>
 #include <math.h>
+#include <netdb.h>
 #include <netinet/in.h>
 #include <netinet/in_systm.h>
 #include <netinet/ip.h>
@@ -55,6 +56,10 @@ extern "C" {
 #endif /* HAVE_IPV6 */
 #include "ResultTable.h"
 #include "TestRecord.h"
+#ifdef HAVE_SNMP
+#include "GetIfInfo.h"
+void PrintIfInfo(const GetIfInfo *);
+#endif /* HAVE_SNMP */
 
 //
 // Default values
@@ -67,12 +72,15 @@ GapType GapDist = GapFixed;
 unsigned int Hops = 30;
 unsigned int Increment = 32;
 unsigned int Mtu = 1500;
-NetworkProtocolType NetworkProtocol = NetworkProtocolIpv4Udp;
+NetworkProtocolType NetworkProtocol = NetworkProtocolNone;
 int NumericFlag = 0;
 unsigned int Port = 32768;
 char *ReadFilename = NULL;
 int QuietFlag = 0;
 unsigned int Repetitions = 32;
+#ifdef HAVE_SNMP
+bool SnmpFlag = false;
+#endif /* HAVE_SNMP */
 unsigned int StartHop = 1;
 unsigned int Timeout = 3;
 unsigned int Tos = 0x0;
@@ -116,7 +124,11 @@ void VersionInfo()
 // Print out invocation information
 //
 void Usage(char *argv0) {
-    fprintf(stderr, "Usage: %s [-a analysis] [-c] [-d debuglevel] [-g gap] [-G gaptype] [-h] [-H hops] [-i increment] [-m mtu] [-n] [-p protocol] [-P port] [-q] [-R reps] [-s hop] [-t timeout] [-T tos] [-v] [-V] [-w file] -r file | host]\n", argv0);
+    fprintf(stderr, "Usage: %s [-a analysis] [-c] [-d debuglevel] [-g gap] [-G gaptype] [-h] [-H hops] [-i increment] [-m mtu] [-n] [-p protocol] [-P port] [-q] [-R reps] [-s hop] "
+#ifdef HAVE_SNMP
+"[-S] "
+#endif /* HAVE_SNMP */
+"[-t timeout] [-T tos] [-v] [-V] [-w file] -r file | host]\n", argv0);
     fprintf(stderr, "\t-a analysis\tSet analysis type (default is lsq)\n");
     fprintf(stderr, "\t\t\tlsq\tLeast squares linear fit\n");
     fprintf(stderr, "\t\t\tkendall\tLinear fit using Kendall's test statistic\n");
@@ -142,6 +154,9 @@ void Usage(char *argv0) {
     fprintf(stderr, "\t-r file\t\tRead data from a file (- for stdin)\n");
     fprintf(stderr, "\t-R reps\t\tRepetitions per hop (default = %d)\n", Repetitions);
     fprintf(stderr, "\t-s hop\t\tStarting hop number (default = %d)\n", StartHop);
+#ifdef HAVE_SNMP
+    fprintf(stderr, "\t-S\t\tDo SNMP queries per-hop\n");
+#endif /* HAVE_SNMP */
     fprintf(stderr, "\t-t timeout\tICMP timeout in seconds (default = %d)\n", Timeout);
     fprintf(stderr, "\t-T tos\t\tSet IP type-of-service field (default = %d)\n", Tos);
     fprintf(stderr, "\t-v\t\tVerbose output\n");
@@ -233,7 +248,7 @@ main(int argc, char **argv)
     FILE *df = NULL;		// output file 
 
     // Parse command-line arguments using getopt
-    while ((c = getopt(argc, argv, "a:cd:g:G:hH:i:m:np:P:qR:r:s:t:T:vVw:")) != -1) {
+    while ((c = getopt(argc, argv, "a:cd:g:G:hH:i:m:np:P:qR:r:s:St:T:vVw:")) != -1) {
 
 	// Check for the different command-line flags we accept
 	switch (c) {
@@ -364,7 +379,7 @@ main(int argc, char **argv)
 		fprintf(stderr, "Couldn't allocate space for statistics filename\n");
 		exit(1);
 	    }
-	    strncpy(ReadFilename, optarg, len);
+	    strncpy(ReadFilename, optarg, len + 1);
 	    if (WriteFilename) {
 		fprintf(stderr, "Warning: both -r and -w specified\n");
 	    }
@@ -387,6 +402,18 @@ main(int argc, char **argv)
 	    break;
 	}
 
+	// S: Enable SNMP queries
+	case 'S': {
+#ifdef HAVE_SNMP
+	    SnmpFlag = true;
+	    break;
+#else
+	    fprintf(stderr, "SNMP unavailable in this build");
+	    Usage(argv[0]);
+	    exit(1);
+#endif /* HAVE_SNMP */
+	}
+
 	// t: ICMP timeout
 	case 't': {
 	    Timeout = atoi(optarg);
@@ -424,7 +451,7 @@ main(int argc, char **argv)
 		fprintf(stderr, "Couldn't allocate space for statistics filename\n");
 		exit(1);
 	    }
-	    strncpy(WriteFilename, optarg, len);
+	    strncpy(WriteFilename, optarg, len + 1);
 	    if (ReadFilename) {
 		fprintf(stderr, "Warning: both -r and -w specified\n");
 	    }
@@ -495,6 +522,72 @@ main(int argc, char **argv)
     }
     else {
 
+	// If the user didn't specify a protocol, we have to figure
+	// out a default.  For IPv4-only, this trivial.
+	if (NetworkProtocol == NetworkProtocolNone) {
+#ifdef HAVE_IPV6
+	    // IPv6 available, so we need to do a little work here.
+	    // We want to resolve the hostname and figure out what
+	    // address family comes back first in the response.
+	    // Presumably if we have IPv6 available, we can do
+	    // getaddrinfo().
+	    struct addrinfo *host = NULL;
+	    struct addrinfo hints;
+	    int error_num;
+
+	    // Setup for getaddrinfo() taken from PctestIpv6::SetTargetName()
+	    memset(&hints, 0, sizeof(hints));
+	    error_num = getaddrinfo(TargetHost, NULL, &hints, &host);
+	    if (host == NULL) {
+
+		// An error here implies there's going to be an error
+		// later, so we might as well blow up now.
+		fprintf(stderr, "%s: %s\n", TargetHost, gai_strerror(error_num));
+		exit(1);
+
+	    }
+
+	    // See which protocol family came back first from the
+	    // resolver query.  We'll use that to determine the
+	    // default protocol to use.
+	    switch (host->ai_family) {
+
+	    case (AF_INET):
+		NetworkProtocol = NetworkProtocolIpv4Udp;
+		IF_DEBUG(1, fprintf(stderr, "Default protocol UDP/IPv4\n"));
+		break;
+
+	    case (AF_INET6):
+		NetworkProtocol = NetworkProtocolIpv6Udp;
+		IF_DEBUG(1, fprintf(stderr, "Default protocol UDP/IPv6\n"));
+		break;
+
+	    default:
+
+		// We don't know what this protocol family is.  Really
+		// there's a better way to handle this.  We should 
+		// traverse the linked list of host->ai_next until
+		// we find a host->ai_family that we *do* recognize,
+		// and then use that, and only snarl at the user if
+		// we ran down the entire chain without finding
+		// something.  Get the simple and dumb behavior
+		// working first.
+		fprintf(stderr, "%s: Unknown protocol family default\n",
+			TargetHost);
+		exit(1);
+		break;
+
+	    }
+
+	    freeaddrinfo(host);
+
+#else
+	    // IPv4 only, so use UDP/IPv4
+	    NetworkProtocol = NetworkProtocolIpv4Udp;
+	    IF_DEBUG(1, fprintf(stderr, "Default protocol UDP/IPv4\n"));
+#endif /* HAVE_IPV6 */	    
+	}
+
 	// Normal case is to make a new protocol-dependent structure
 	switch (NetworkProtocol) {
 	case (NetworkProtocolIpv4Udp):
@@ -638,6 +731,15 @@ main(int argc, char **argv)
 		    fprintf(stdout, "(%s)", pct->GetName(tr->icmpDestAddress));
 		}
 		fprintf(stdout, "\n");
+#ifdef HAVE_SNMP
+		if (!ReadFilename && SnmpFlag) {
+		    GetIfInfo *gifp;
+		    gifp = new GetIfInfo(tr->icmpDestAddress, pct);
+		    PrintIfInfo(gifp);
+		    delete gifp;
+		}
+#endif /* HAVE_SNMP */
+
 	    }
 
 	}
@@ -949,7 +1051,7 @@ endreps:
 	int qBytes;		// estimate of bytes queued along path
 	qTime = PartialResults[i]->queueing();
 	qTimeHop = qTime - queueingTime;
-	if ((hopBandwidth >= 0.0) && (qTimeHop >= 0.0)) {
+	if ((hopBandwidth > 0.0) && (qTimeHop >= 0.0)) {
 	    qBytes = (int) (qTimeHop * (1.0/bHop));
 	}
 	else {
@@ -996,6 +1098,16 @@ endreps:
 		fprintf(stdout, "(%s)", pct->GetName(checkAddress));
 	    }
 	    fprintf(stdout, "\n");
+
+#ifdef HAVE_SNMP
+	    if (!ReadFilename && SnmpFlag) {
+		GetIfInfo *gifp;
+		gifp = new GetIfInfo(checkAddress, pct);
+		PrintIfInfo(gifp);
+		delete gifp;
+	    }
+#endif /* HAVE_SNMP */
+
 	}
 
 	// Update inter-hop state
@@ -1048,3 +1160,22 @@ endreps:
 
 }
 
+#ifdef HAVE_SNMP
+void PrintIfInfo(const GetIfInfo *gifp)
+{
+    if (gifp == NULL) {
+	return;
+    }
+
+    fprintf(stdout, "    Description:       %s\n", gifp->GetDescription());
+    fprintf(stdout, "    Name:              %s\n", gifp->GetName());
+    fprintf(stdout, "    Contact:           %s\n", gifp->GetContact());
+    fprintf(stdout, "    Location:          %s\n", gifp->GetLocation());
+    fprintf(stdout, "    IfDescription:     %s\n", gifp->GetIfDescription());
+    fprintf(stdout, "    Interface:         type = %s(%lu)\n",
+	    gifp->GetIfTypeString(), gifp->GetIfType());
+    fprintf(stdout, "                       speed = %lu bps, MTU = %lu\n",
+	    gifp->GetIfSpeed(), gifp->GetIfMtu());
+    fflush(stdout);
+}
+#endif /* HAVE_SNMP */
